@@ -1,6 +1,7 @@
 # debhelper format -- lintian check script -*- perl -*-
 
-# Copyright (C) 1999 by Joey Hess
+# Copyright © 1999 by Joey Hess
+# Copyright © 2016-2020 Chris Lamb <lamby@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,12 +26,15 @@ use warnings;
 use utf8;
 use autodie;
 
-use List::MoreUtils qw(any);
-use Text::Levenshtein qw(distance);
+use List::Compare;
+use List::MoreUtils qw(any firstval);
+use List::UtilsBy qw(min_by);
+use Text::LevenshteinXS qw(distance);
 
 use Lintian::Data;
 use Lintian::Relation qw(:constants);
-use Lintian::Util qw(strip);
+
+use constant UNDERSCORE => q{_};
 
 use Moo;
 use namespace::clean;
@@ -46,14 +50,18 @@ my $cdbscompat = 5;
 my $maint_commands = Lintian::Data->new('debhelper/maint_commands');
 my $dh_commands_depends = Lintian::Data->new('debhelper/dh_commands', '=');
 my $filename_configs = Lintian::Data->new('debhelper/filename-config-files');
-my $dh_ver_deps= Lintian::Data->new('debhelper/dh_commands-manual', qr/\|\|/o);
+my $dh_ver_deps= Lintian::Data->new('debhelper/dh_commands-manual', qr/\|\|/);
 my $dh_addons = Lintian::Data->new('common/dh_addons', '=');
 my $dh_addons_manual
-  = Lintian::Data->new('debhelper/dh_addons-manual', qr/\|\|/o);
+  = Lintian::Data->new('debhelper/dh_addons-manual', qr/\|\|/);
 my $compat_level = Lintian::Data->new('debhelper/compat-level',qr/=/);
 
 my $MISC_DEPENDS = Lintian::Relation->new('${misc:Depends}');
 
+my @KNOWN_DH_COMMANDS= map { $_, $_ . '-arch', $_ . '-indep' }
+  map { 'override' . $_, 'execute_before' . $_, 'execute_after' . $_ }
+  map { UNDERSCORE . $_ } $dh_commands_depends->all;
+
 sub source {
     my ($self) = @_;
 
@@ -73,9 +81,10 @@ sub source {
     my $inclcdbs = 0;
 
     my ($bdepends_noarch, $bdepends, %build_systems, $uses_autotools_dev_dh);
-    $bdepends = $processable->relation('build-depends-all');
+    $bdepends = $processable->relation('Build-Depends-All');
     my $seen_dh = 0;
     my $seen_dh_dynamic = 0;
+    my $seen_dh_systemd = 0;
     my $seen_dh_parallel = 0;
     my %seen = (
         'python2' => 0,
@@ -163,9 +172,9 @@ sub source {
             $needbuilddepends = 1;
             $needtomodifyscripts = 1;
 
-            while (m/\s--with(?:=|\s+)(['"]?)(\S+)\1/go) {
+            while (m/\s--with(?:=|\s+)(['"]?)(\S+)\1/g) {
                 my $addon_list = $2;
-                for my $addon (split(m/,/o, $addon_list)) {
+                for my $addon (split(/,/, $addon_list)) {
                     my $orig_addon = $addon;
                     $addon =~ y,-,_,;
                     my $depends = $dh_addons_manual->value($addon)
@@ -177,13 +186,15 @@ sub source {
                         );
                         $uses_autotools_dev_dh = 1;
                     }
+                    $seen_dh_systemd = $. if $addon eq 'systemd';
                     $self->tag(
                         'dh-quilt-addon-but-quilt-source-format',
                         "dh ... --with ${orig_addon}",
                         "(line $.)"
                       )
                       if $addon eq 'quilt'
-                      and $processable->field('format', '') eq '3.0 (quilt)';
+                      and $processable->fields->value('Format') eq
+                      '3.0 (quilt)';
                     if (defined $depends) {
                         $missingbdeps_addons{$depends} = $addon;
                     }
@@ -193,7 +204,7 @@ sub source {
                 }
             }
         } elsif (m,^include\s+/usr/share/cdbs/1/rules/debhelper.mk,
-            or m,^include\s+/usr/share/R/debian/r-cran.mk,o) {
+            or m,^include\s+/usr/share/R/debian/r-cran.mk,) {
             $build_systems{'cdbs-with-debhelper.mk'} = 1;
             delete($build_systems{'cdbs-without-debhelper.mk'});
             $seencommand = 1;
@@ -211,28 +222,60 @@ sub source {
             $dhcompatvalue = $1;
             # one can export and then set the value:
             $level = $1 if ($level);
-        } elsif (/^([^:]*override_dh_[^:]*):/) {
-            my $targets = $1;
-            $needbuilddepends = 1;
-            # Can be multiple targets per rule.
-            while ($targets =~ /\boverride_dh_([^\s]+?)(-arch|-indep|)\b/g) {
-                my $cmd = $1;
-                my $arch = $2;
+
+        } elsif (/^[^:]*(override|execute_(?:after|before))\s+(dh_[^:]*):/) {
+            $self->tag('typo-in-debhelper-override-target',
+                "$1 $2", '->', "$1_$2","(line $.)");
+
+        } elsif (/^([^:]*_dh_[^:]*):/) {
+            my $alltargets = $1;
+            # can be multiple targets per rule.
+            my @targets = split(/\s+/, $alltargets);
+            my @dh_targets = grep { /_dh_/ } @targets;
+
+            # If maintainer is using wildcards, it's unlikely to be a typo.
+            my @no_wildcards = grep { !/%/ } @dh_targets;
+
+            my $lc = List::Compare->new(\@no_wildcards, \@KNOWN_DH_COMMANDS);
+            my @unknown = $lc->get_Lonly;
+
+            for my $target (@unknown) {
+
+                my %distance
+                  = map { $_ => distance($target, $_) } @KNOWN_DH_COMMANDS;
+                my @near = grep { $distance{$_} < 3 } keys %distance;
+                my $nearest = min_by { $distance{$_} } @near;
+
+                $self->tag('typo-in-debhelper-override-target',
+                    $target, '->', $nearest, "(line $.)")
+                  if length $nearest;
+            }
+
+            for my $target (@no_wildcards) {
+
+                next
+                  unless $target
+                  =~ /^(override|execute_(?:before|after))_dh_([^\s]+?)(-arch|-indep|)$/;
+
+                my $prefix = $1;
+                my $cmd = $2;
+                my $arch = $3;
                 my $dhcommand = "dh_$cmd";
                 $overrides{$dhcommand} = [$., $arch];
-                # If maintainer is using wildcards, it's unlikely to be a typo.
-                next if ($dhcommand =~ /%/);
-                next if ($dh_commands_depends->known($dhcommand));
+                $needbuilddepends = 1;
+
+                next
+                  if $dh_commands_depends->known($dhcommand);
+
                 # Unknown command, so check for likely misspellings
-                foreach my $x (sort $dh_commands_depends->all) {
-                    if ("dh_auto_$cmd" eq $x or distance($dhcommand, $x) < 3) {
-                        $self->tag('typo-in-debhelper-override-target',
-                            "override_$dhcommand", '->', "override_$x",
-                            "(line $.)");
-                        last; # Only emit a single match
-                    }
-                }
+                my $missingauto = firstval { "dh_auto_$cmd" eq $_ }
+                $dh_commands_depends->all;
+                $self->tag('typo-in-debhelper-override-target',
+                    "${prefix}_$dhcommand", '->', "${prefix}_$missingauto",
+                    "(line $.)")
+                  if length $missingauto;
             }
+
         } elsif (m,^include\s+/usr/share/cdbs/,) {
             $inclcdbs = 1;
             $build_systems{'cdbs-without-debhelper.mk'} = 1
@@ -286,13 +329,16 @@ sub source {
         return;
     }
 
-    my @pkgs = $processable->binaries;
+    my @pkgs = $processable->debian_control->installables;
     my $single_pkg = '';
-    $single_pkg =  $processable->binary_package_type($pkgs[0])
+    $single_pkg
+      =  $processable->debian_control->installable_package_type($pkgs[0])
       if scalar @pkgs == 1;
 
     for my $binpkg (@pkgs) {
-        next if $processable->binary_package_type($binpkg) ne 'deb';
+        next
+          if $processable->debian_control->installable_package_type($binpkg)ne
+          'deb';
         my $strong = $processable->binary_relation($binpkg, 'strong');
         my $all = $processable->binary_relation($binpkg, 'all');
 
@@ -306,7 +352,7 @@ sub source {
 
     for my $proc ($group->get_processables('binary')) {
         my $binpkg = $proc->name;
-        my $breaks = $processable->binary_relation($binpkg, 'breaks');
+        my $breaks = $processable->binary_relation($binpkg, 'Breaks');
         my $strong = $processable->binary_relation($binpkg, 'strong');
         $self->tag('package-uses-dh-runit-but-lacks-breaks-substvar', $binpkg)
           if $seen{'runit'}
@@ -334,7 +380,11 @@ sub source {
         open(my $fd, '<', $compat_file->unpacked_path);
         while (<$fd>) {
             if ($. == 1) {
-                $compat = strip($_);
+                $compat = $_;
+
+                # trim both ends
+                $compat =~ s/^\s+|\s+$//g;
+
             } elsif (m/^\d/) {
                 $self->tag('debhelper-compat-file-contains-multiple-levels',
                     "(line $.)");
@@ -413,6 +463,11 @@ sub source {
         "(line $seen_dh_parallel)"
     ) if $seen_dh_parallel and $level >= 10;
 
+    $self->tag(
+        'debian-rules-uses-unnecessary-dh-argument',
+        "dh ... --with=systemd (line $seen_dh_systemd)"
+    ) if $seen_dh_systemd and $level >= 10;
+
     # Check the files in the debian directory for various debhelper-related
     # things.
     for my $file ($droot->children) {
@@ -437,8 +492,9 @@ sub source {
             }
             close($fd);
             if (!$seentag) {
-                my $binpkg_type = $processable->binary_package_type($binpkg)
-                  // 'deb';
+                my $binpkg_type
+                  = $processable->debian_control->installable_package_type(
+                    $binpkg);
                 my $is_udeb = 0;
                 $is_udeb = 1 if $binpkg and $binpkg_type eq 'udeb';
                 $is_udeb = 1 if not $binpkg and $single_pkg eq 'udeb';
@@ -448,7 +504,7 @@ sub source {
                 }
             }
         } elsif ($basename eq 'control'
-            or $basename =~ m/^(?:.*\.)?(?:copyright|changelog|NEWS)$/o) {
+            or $basename =~ m/^(?:.*\.)?(?:copyright|changelog|NEWS)$/) {
             # Handle "control", [<pkg>.]copyright, [<pkg>.]changelog
             # and [<pkg>.]NEWS
             $self->tag_if_executable($file);
@@ -522,8 +578,8 @@ sub source {
         }
     }
 
-    $bdepends_noarch = $processable->relation_noarch('build-depends-all');
-    $bdepends = $processable->relation('build-depends-all');
+    $bdepends_noarch = $processable->relation_noarch('Build-Depends-All');
+    $bdepends = $processable->relation('Build-Depends-All');
     if ($needbuilddepends) {
         $self->tag('package-uses-debhelper-but-lacks-build-depends')
           unless $bdepends->implies('debhelper')
@@ -712,9 +768,10 @@ sub _shebang_cmd {
             # It is beyond me why anyone would place a lincity data
             # file here...  but if they do, we will handle it
             # correctly.
-            $cmd = '' if $cmd =~ m/^#!/o;
+            $cmd = '' if $cmd =~ /^#!/;
 
-            strip($cmd);
+            # trim both ends
+            $cmd =~ s/^\s+|\s+$//g;
         }
     }
     close($fd);
