@@ -24,14 +24,14 @@ use v5.20;
 use warnings;
 use utf8;
 
-use Capture::Tiny qw(capture);
+use Lintian::IPC::Run3 qw(safe_qx);
 
 use Moo;
 use namespace::clean;
 
 with 'Lintian::Check';
 
-sub files {
+sub visit_installed_files {
     my ($self, $file) = @_;
 
     return
@@ -39,10 +39,10 @@ sub files {
 
     if ($file->name =~ /\.xz$/si) {
 
-        capture {
-            $self->tag('broken-xz', $file->name)
-              if system('xz', '--test', $file->unpacked_path);
-        };
+        safe_qx('xz', '--test', $file->unpacked_path);
+
+        $self->tag('broken-xz', $file->name)
+          if $?;
     }
 
     return;
