@@ -1,6 +1,6 @@
 # files/ld-so -- lintian check script -*- perl -*-
 
-# Copyright (C) 1998 Christian Schwarz and Richard Braakman
+# Copyright © 1998 Christian Schwarz and Richard Braakman
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,11 +30,11 @@ use namespace::clean;
 
 with 'Lintian::Check';
 
-sub files {
+sub visit_installed_files {
     my ($self, $file) = @_;
 
     if (    $file->name =~ m,^etc/ld\.so\.conf\.d/.+$,
-        and $self->package !~ /^libc/){
+        and $self->processable->name !~ /^libc/){
         $self->tag('package-modifies-ld.so-search-path', $file->name);
     }
 
