@@ -1,6 +1,6 @@
 # files/pam -- lintian check script -*- perl -*-
 
-# Copyright (C) 1998 Christian Schwarz and Richard Braakman
+# Copyright © 1998 Christian Schwarz and Richard Braakman
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,12 +30,13 @@ use namespace::clean;
 
 with 'Lintian::Check';
 
-sub files {
+sub visit_installed_files {
     my ($self, $file) = @_;
 
     # /etc/pam.conf
     $self->tag('config-file-reserved', $file->name, 'by libpam-runtime')
-      if $file->name =~ m,^etc/pam.conf, && $self->package ne 'libpam-runtime';
+      if $file->name =~ m,^etc/pam.conf,
+      && $self->processable->name ne 'libpam-runtime';
 
     return;
 }
