@@ -1,6 +1,7 @@
 # menu format -- lintian check script -*- perl -*-
 
-# Copyright (C) 1998 by Joey Hess
+# Copyright © 1998 by Joey Hess
+# Copyright © 2017-2018 Chris Lamb <lamby@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -33,15 +34,21 @@
 #  - Review desktop-file-validate to see what else we're missing.
 
 package Lintian::menu_format;
-use strict;
+
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
 use File::Basename;
 use List::MoreUtils qw(any);
 
 use Lintian::Data;
-use Lintian::Tags qw(tag);
+
+use Moo;
+use namespace::clean;
+
+with 'Lintian::Check';
 
 # This is a list of all tags that should be in every menu item.
 my @req_tags = qw(needs section title command);
@@ -152,11 +159,17 @@ my %needs_tag_vals_hash = map { $_ => 1 } @needs_tag_vals;
 
 # -----------------------------------
 
-sub binary {
-    my ($pkg, $type, $info, $proc, $group) = @_;
+sub installable {
+    my ($self) = @_;
+
+    my $pkg = $self->processable->name;
+    my $type = $self->processable->type;
+    my $processable = $self->processable;
+    my $group = $self->group;
+
     my (@menufiles, %desktop_cmds);
     for my $dirname (qw(usr/share/menu/ usr/lib/menu/)) {
-        if (my $dir = $info->index_resolved_path($dirname)) {
+        if (my $dir = $processable->installed->resolve_path($dirname)) {
             push(@menufiles, $dir->children);
         }
     }
@@ -164,13 +177,13 @@ sub binary {
     # Find the desktop files in the package for verification.
     my @desktop_files;
     for my $subdir (qw(applications xsessions)) {
-        if (my $dir = $info->index("usr/share/$subdir/")) {
+        if (my $dir = $processable->installed->lookup("usr/share/$subdir/")) {
             for my $file ($dir->children) {
                 next unless $file->is_file;
-                next unless $file->basename =~ m/\.desktop$/;
+                next unless $file->basename =~ m/\.desktop$/ && !$file->is_dir;
                 if ($file->is_executable) {
-                    tag 'executable-desktop-file',
-                      sprintf('%s %04o',$file, $file->operm);
+                    $self->tag('executable-desktop-file',
+                        sprintf('%s %04o',$file, $file->operm));
                 }
                 if (index($file, 'template') == -1) {
                     push(@desktop_files, $file);
@@ -181,7 +194,7 @@ sub binary {
 
     # Verify all the desktop files.
     for my $desktop_file (@desktop_files) {
-        verify_desktop_file($desktop_file, $pkg, $info, \%desktop_cmds);
+        $self->verify_desktop_file($desktop_file, \%desktop_cmds);
     }
 
     # Now all the menu files.
@@ -192,9 +205,9 @@ sub binary {
         my $fullname = $menufile->name;
 
         # README is a special case
-        next if $menufile->basename eq 'README';
+        next if $menufile->basename eq 'README' && !$menufile->is_dir;
         my $menufile_line ='';
-        my $fd = $menufile->open;
+        open(my $fd, '<', $menufile->unpacked_path);
         # line below is commented out in favour of the while loop
         # do { $_=<IN>; } while defined && (m/^\s* \#/ || m/^\s*$/);
         while (<$fd>) {
@@ -208,7 +221,7 @@ sub binary {
 
         # Check first line of file to see if it matches the new menu
         # file format.
-        if ($menufile_line =~ m/^!C\s*menu-2/o) {
+        if ($menufile_line =~ m/^!C\s*menu-2/) {
             # we can't parse that yet
             close($fd);
             next;
@@ -221,26 +234,19 @@ sub binary {
             $lc++;
 
             # Ignore lines that are comments.
-            if ($menufile_line =~ m/^\s*\#/o) {
+            if ($menufile_line =~ m/^\s*\#/) {
                 next;
             }
             $line .= $menufile_line;
             # Note that I allow whitespace after the continuation character.
             # This is caught by verify_line().
             if (!($menufile_line =~ m/\\\s*?$/)) {
-                verify_line(
-                    $pkg, $info, $proc, $group,
-                    $type,$menufile, $fullname, $line,
-                    $lc,\%desktop_cmds
-                );
+                $self->verify_line($menufile, $fullname, $line,
+                    $lc,\%desktop_cmds);
                 $line='';
             }
         } while ($menufile_line = <$fd>);
-        verify_line(
-            $pkg, $info, $proc, $group,
-            $type,$menufile, $fullname, $line,
-            $lc,\%desktop_cmds
-        );
+        $self->verify_line($menufile, $fullname, $line,$lc,\%desktop_cmds);
 
         close($fd);
     }
@@ -253,10 +259,12 @@ sub binary {
 # Pass this a line of a menu file, it sanitizes it and
 # verifies that it is correct.
 sub verify_line {
-    my (
-        $pkg, $info, $proc, $group, $type,
-        $menufile, $fullname, $line, $linecount,$desktop_cmds
-    ) = @_;
+    my ($self, $menufile, $fullname, $line, $linecount,$desktop_cmds) = @_;
+
+    my $pkg = $self->processable->name;
+    my $type = $self->processable->type;
+    my $processable = $self->processable;
+    my $group = $self->group;
 
     my %vals;
 
@@ -264,30 +272,31 @@ sub verify_line {
 
     # Replace all line continuation characters with whitespace.
     # (do not remove them completely, because update-menus doesn't)
-    $line =~ s/\\\n/ /mgo;
+    $line =~ s/\\\n/ /mg;
 
     # This is in here to fix a common mistake: whitespace after a '\'
     # character.
-    if ($line =~ s/\\\s+\n/ /mgo) {
-        tag 'whitespace-after-continuation-character', "$fullname:$linecount";
+    if ($line =~ s/\\\s+\n/ /mg) {
+        $self->tag('whitespace-after-continuation-character',
+            "$fullname:$linecount");
     }
 
     # Ignore lines that are all whitespace or empty.
-    return if $line =~ m/^\s*$/o;
+    return if $line =~ m/^\s*$/;
 
     # Ignore lines that are comments.
-    return if $line =~ m/^\s*\#/o;
+    return if $line =~ m/^\s*\#/;
 
     # Start by testing the package check.
-    if (not $line =~ m/^\?package\((.*?)\):/o) {
-        tag 'bad-test-in-menu-item', "$fullname:$linecount";
+    if (not $line =~ m/^\?package\((.*?)\):/) {
+        $self->tag('bad-test-in-menu-item', "$fullname:$linecount");
         return;
     }
     my $pkg_test = $1;
     my %tested_packages = map { $_ => 1 } split(/\s*,\s*/, $pkg_test);
     my $tested_packages = scalar keys %tested_packages;
     unless (exists $tested_packages{$pkg}) {
-        tag 'pkg-not-in-package-test', "$pkg_test $fullname";
+        $self->tag('pkg-not-in-package-test', "$pkg_test $fullname");
     }
     $line =~ s/^\?package\(.*?\)://;
 
@@ -323,20 +332,21 @@ sub verify_line {
              *                  # match as many times as we can
             )
            )
-           /ogcx
+           /gcx
     ) {
         my $tag = $1;
         my $value = $2;
 
         if (exists $vals{$tag}) {
-            tag 'duplicated-tag-in-menu-item', "$fullname $1:$linecount";
+            $self->tag('duplicate-tag-in-menu',"$fullname $1:$linecount");
         }
 
         # If the value was quoted, remove those quotes.
         if ($value =~ m/^\"(.*)\"$/) {
             $value = $1;
         } else {
-            tag 'unquoted-string-in-menu-item', "$fullname $1:$linecount";
+            $self->tag('unquoted-string-in-menu-item',
+                "$fullname $1:$linecount");
         }
 
         # If the value has escaped characters, remove the
@@ -353,12 +363,12 @@ sub verify_line {
     # We use this below when we look at how far along in the
     # string we matched. So the point of this line is to allow
     # trailing whitespace on the end of a line.
-    $line =~ m/\s*/ogc;
+    $line =~ m/\s*/gc;
 
     # If that loop didn't match up to end of line, we have a
     # problem..
     if (pos($line) < length($line)) {
-        tag 'unparsable-menu-item', "$fullname:$linecount";
+        $self->tag('unparsable-menu-item', "$fullname:$linecount");
         # Give up now, before things just blow up in our face.
         return;
     }
@@ -368,7 +378,10 @@ sub verify_line {
     # Test for important tags.
     foreach my $tag (@req_tags) {
         unless (exists($vals{$tag}) && defined($vals{$tag})) {
-            tag 'menu-item-missing-required-tag', "$tag $fullname:$linecount";
+            $self->tag(
+                'menu-item-missing-required-tag',
+                "$tag $fullname:$linecount"
+            );
             # Just give up right away, if such an essential tag is missing,
             # chance is high the rest doesn't make sense either. And now all
             # following checks can assume those tags to be there
@@ -379,7 +392,10 @@ sub verify_line {
     # Make sure all tags are known.
     foreach my $tag (keys %vals) {
         if (!$known_tags_hash{$tag}) {
-            tag 'menu-item-contains-unknown-tag', "$tag $fullname:$linecount";
+            $self->tag(
+                'menu-item-contains-unknown-tag',
+                "$tag $fullname:$linecount"
+            );
         }
     }
 
@@ -391,8 +407,8 @@ sub verify_line {
 
     # Be sure the command is provided by the package.
     my ($okay, $command)
-      = verify_cmd($fullname, $linecount, $vals{'command'},$pkg, $info);
-    tag 'menu-command-not-in-package', "$fullname:$linecount $command"
+      = $self->verify_cmd($fullname, $linecount, $vals{'command'});
+    $self->tag('menu-command-not-in-package', "$fullname:$linecount $command")
       unless ($okay
         or not $command
         or ($tested_packages >= 2)
@@ -402,21 +418,20 @@ sub verify_line {
     if (defined($command)) {
         $command =~ s@^(?:usr/)?s?bin/@@;
         if ($desktop_cmds->{$command}) {
-            tag 'command-in-menu-file-and-desktop-file', $command,
-              "${fullname}:${linecount}";
+            $self->tag('command-in-menu-file-and-desktop-file',
+                $command,"${fullname}:${linecount}");
         }
     }
 
     if (exists($vals{'icon'})) {
-        verify_icon($info, $proc, $group, $menufile, $fullname, $linecount,
-            $vals{'icon'}, 32);
+        $self->verify_icon($menufile, $fullname, $linecount,$vals{'icon'}, 32);
     }
     if (exists($vals{'icon32x32'})) {
-        verify_icon($info, $proc, $group, $menufile, $fullname, $linecount,
+        $self->verify_icon($menufile, $fullname, $linecount,
             $vals{'icon32x32'}, 32);
     }
     if (exists($vals{'icon16x16'})) {
-        verify_icon($info, $proc, $group, $menufile, $fullname, $linecount,
+        $self->verify_icon($menufile, $fullname, $linecount,
             $vals{'icon16x16'}, 16);
     }
 
@@ -426,37 +441,38 @@ sub verify_line {
     if ($section =~ m:^(WindowManagers/Modules|FVWM Modules|Window Maker):) {
         # WM/Modules: needs must not be the regular ones nor wm
         if ($needs_tag_vals_hash{$needs} or $needs eq 'wm') {
-            tag 'non-wm-module-in-wm-modules-menu-section',
-              "$needs $fullname:$linecount";
+            $self->tag('non-wm-module-in-wm-modules-menu-section',
+                "$needs $fullname:$linecount");
         }
     } elsif ($section =~ m:^Window ?Managers:) {
         # Other WM sections: needs must be wm
         if ($needs ne 'wm') {
-            tag 'non-wm-in-windowmanager-menu-section',
-              "$needs $fullname:$linecount";
+            $self->tag('non-wm-in-windowmanager-menu-section',
+                "$needs $fullname:$linecount");
         }
     } else {
         # Any other section: just only the general ones
         if ($needs eq 'dwww') {
-            tag 'menu-item-needs-dwww', "$fullname:$linecount";
+            $self->tag('menu-item-needs-dwww', "$fullname:$linecount");
         } elsif (not $needs_tag_vals_hash{$needs}) {
-            tag 'menu-item-needs-tag-has-unknown-value',
-              "$needs $fullname:$linecount";
+            $self->tag('menu-item-needs-tag-has-unknown-value',
+                "$needs $fullname:$linecount");
         }
     }
 
     # Check the section tag
     # Check for historical changes in the section tree.
     if ($section =~ m:^Apps/Games:) {
-        tag 'menu-item-uses-apps-games-section', "$fullname:$linecount";
+        $self->tag('menu-item-uses-apps-games-section',"$fullname:$linecount");
         $section =~ s:^Apps/::;
     }
     if ($section =~ m:^Apps/:) {
-        tag 'menu-item-uses-apps-section', "$fullname:$linecount";
+        $self->tag('menu-item-uses-apps-section', "$fullname:$linecount");
         $section =~ s:^Apps/:Applications/:;
     }
     if ($section =~ m:^WindowManagers:) {
-        tag 'menu-item-uses-windowmanagers-section', "$fullname:$linecount";
+        $self->tag('menu-item-uses-windowmanagers-section',
+            "$fullname:$linecount");
         $section =~ s:^WindowManagers:Window Managers:;
     }
 
@@ -465,8 +481,10 @@ sub verify_line {
     my $root_data = $MENU_SECTIONS->value($rootsec);
     if (not defined $root_data) {
         if (not $rootsec =~ m/$pkg/i) {
-            tag 'menu-item-creates-new-root-section',
-              "$rootsec $fullname:$linecount";
+            $self->tag(
+                'menu-item-creates-new-root-section',
+                "$rootsec $fullname:$linecount"
+            );
         }
     } else {
         my $ok = 1;
@@ -478,58 +496,63 @@ sub verify_line {
             $ok = 0 if %$root_data;
         }
         unless ($ok) {
-            tag 'menu-item-creates-new-section',
-              "$vals{section} $fullname:$linecount";
+            $self->tag(
+                'menu-item-creates-new-section',
+                "$vals{section} $fullname:$linecount"
+            );
         }
     }
     return;
 }
 
 sub verify_icon {
-    my ($info, $proc, $group, $menufile, $fullname, $linecount, $icon, $size)
-      = @_;
+    my ($self, $menufile, $fullname, $linecount, $icon, $size)= @_;
+
+    my $processable = $self->processable;
+    my $group = $self->group;
 
     if ($icon eq 'none') {
-        tag 'menu-item-uses-icon-none', "$fullname:$linecount";
+        $self->tag('menu-item-uses-icon-none', "$fullname:$linecount");
         return;
     }
 
-    tag 'menu-icon-uses-relative-path', $icon
-      unless $icon =~ m,^/,;
+    $self->tag('menu-icon-uses-relative-path', $icon) unless $icon =~ m,^/,;
 
     $icon =~ s|^/*||og;
 
     if (not($icon =~ m/\.xpm$/i)) {
-        tag 'menu-icon-not-in-xpm-format', $icon;
+        $self->tag('menu-icon-not-in-xpm-format', $icon);
         return;
     }
 
     # Try the explicit location, and if that fails, try the standard path.
-    my $iconfile = $info->index_resolved_path($icon);
+    my $iconfile = $processable->installed->resolve_path($icon);
     if (not $iconfile) {
-        $iconfile = $info->index_resolved_path("usr/share/pixmaps/$icon");
+        $iconfile
+          = $processable->installed->resolve_path("usr/share/pixmaps/$icon");
         if (not $iconfile) {
-            my $ginfo = $group->info;
-            foreach my $depproc (@{ $ginfo->direct_dependencies($proc) }) {
-                my $dinfo = $depproc->info;
-                $iconfile = $dinfo->index_resolved_path($icon);
+            foreach
+              my $depproc (@{ $group->direct_dependencies($processable) }) {
+
+                $iconfile = $depproc->installed->resolve_path($icon);
                 last if $iconfile;
                 $iconfile
-                  = $dinfo->index_resolved_path("usr/share/pixmaps/$icon");
+                  = $depproc->installed->resolve_path(
+                    "usr/share/pixmaps/$icon");
                 last if $iconfile;
             }
         }
     }
 
     if (not $iconfile or not $iconfile->is_open_ok) {
-        tag 'menu-icon-missing', $icon;
+        $self->tag('menu-icon-missing', $icon);
         return;
     }
 
     my $parse = 'XPM header';
     my $line;
 
-    my $fd = $iconfile->open;
+    open(my $fd, '<', $iconfile->unpacked_path);
 
     do { defined($line = <$fd>) or goto parse_error; }
       until ($line =~ /\/\*\s*XPM\s*\*\//);
@@ -541,7 +564,8 @@ sub verify_icon {
     my $height = $2 + 0;
 
     if ($width > $size || $height > $size) {
-        tag 'menu-icon-too-big', "$icon: ${width}x${height} > ${size}x${size}";
+        $self->tag('menu-icon-too-big',
+            "$icon: ${width}x${height} > ${size}x${size}");
     }
 
     close($fd);
@@ -549,20 +573,24 @@ sub verify_icon {
 
   parse_error:
     close($fd);
-    tag 'menu-icon-cannot-be-parsed', "$icon: looking for $parse";
+    $self->tag('menu-icon-cannot-be-parsed', "$icon: looking for $parse");
     return;
 }
 
 # Syntax-checks a .desktop file.
 sub verify_desktop_file {
-    my ($file, $pkg, $info, $desktop_cmds) = @_;
+    my ($self, $file, $desktop_cmds) = @_;
+
+    my $pkg = $self->processable->name;
+
     my ($saw_first, $warned_cr, %vals, @pending);
-    my $fd = $file->open;
+    open(my $fd, '<', $file->unpacked_path);
     while (my $line = <$fd>) {
         chomp $line;
         next if ($line =~ m/^\s*\#/ or $line =~ m/^\s*$/);
         if ($line =~ s/\r//) {
-            tag 'desktop-entry-file-has-crs', "$file:$." unless $warned_cr;
+            $self->tag('desktop-entry-file-has-crs', "$file:$.")
+              unless $warned_cr;
             $warned_cr = 1;
         }
 
@@ -573,7 +601,7 @@ sub verify_desktop_file {
         unless ($saw_first) {
             return unless $line =~ /^\[(KDE )?Desktop Entry\]\s*$/;
             $saw_first = 1;
-            tag 'desktop-contains-deprecated-key', "$file:$."
+            $self->tag('desktop-contains-deprecated-key', "$file:$.")
               if ($line =~ /^\[KDE Desktop Entry\]\s*$/);
         }
 
@@ -589,7 +617,7 @@ sub verify_desktop_file {
             my $basetag = $tag;
             $basetag =~ s/\[([^\]]+)\]$//;
             if (exists $vals{$tag}) {
-                tag 'duplicated-key-in-desktop-entry', "$file:$. $tag";
+                $self->tag('duplicate-key-in-desktop', "$file:$. $tag");
             } elsif ($DEPRECATED_DESKTOP_KEYS->known($basetag)) {
                 if ($basetag eq 'Encoding') {
                     push(
@@ -618,27 +646,35 @@ sub verify_desktop_file {
     close($fd);
 
     # Now validate the data in the desktop file, but only if it's a known type.
-    return unless ($vals{'Type'} and $known_desktop_types{$vals{'Type'}});
+    # Warn if it's not.
+    my $type = $vals{'Type'};
+    return
+      unless defined $type;
+
+    unless ($known_desktop_types{$type}) {
+        $self->tag('desktop-entry-unknown-type', $file, $type);
+        return;
+    }
 
     # Now we can issue any pending tags.
     for my $pending (@pending) {
-        tag @$pending;
+        $self->tag(@$pending);
     }
 
     # Test for important keys.
     for my $tag (@req_desktop_keys) {
         unless (defined $vals{$tag}) {
-            tag 'desktop-entry-missing-required-key', "$file $tag";
+            $self->tag('desktop-entry-missing-required-key', "$file $tag");
         }
     }
 
     # test if missing Keywords (only if NoDisplay is not set)
     if (!defined $vals{NoDisplay}) {
         if (!defined $vals{Icon}) {
-            tag 'desktop-entry-lacks-icon-entry', $file;
+            $self->tag('desktop-entry-lacks-icon-entry', $file);
         }
         if (!defined $vals{Keywords} && $vals{'Type'} eq 'Application') {
-            tag 'desktop-entry-lacks-keywords-entry', $file;
+            $self->tag('desktop-entry-lacks-keywords-entry', $file);
         }
     }
 
@@ -652,8 +688,8 @@ sub verify_desktop_file {
         and $vals{'Exec'}
         and $vals{'Exec'} =~ /\S/) {
         my ($okay, $command)
-          = verify_cmd($file->name, undef, $vals{'Exec'}, $pkg,$info);
-        tag 'desktop-command-not-in-package', $file, $command
+          = $self->verify_cmd($file->name, undef, $vals{'Exec'});
+        $self->tag('desktop-command-not-in-package', $file, $command)
           unless $okay
           or $command eq 'kcmshell';
         $command =~ s@^(?:usr/)?s?bin/@@;
@@ -669,19 +705,19 @@ sub verify_desktop_file {
         for my $cat (@cats) {
             next if $cat =~ /^X-/;
             if ($reserved_categories{$cat}) {
-                tag 'desktop-entry-uses-reserved-category', "$cat $file"
+                $self->tag('desktop-entry-uses-reserved-category',"$cat $file")
                   unless $vals{'OnlyShowIn'};
                 $saw_main = 1;
                 $in_reserved = 1;
             } elsif (not $ADD_CATEGORIES->known($cat)
                 and not $main_categories{$cat}) {
-                tag 'desktop-entry-invalid-category', "$cat $file";
+                $self->tag('desktop-entry-invalid-category', "$cat $file");
             } elsif ($main_categories{$cat}) {
                 $saw_main = 1;
             }
         }
         unless ($saw_main) {
-            tag 'desktop-entry-lacks-main-category', $file;
+            $self->tag('desktop-entry-lacks-main-category', $file);
         }
     }
 
@@ -692,7 +728,7 @@ sub verify_desktop_file {
     if (defined $vals{OnlyShowIn} and not $in_reserved) {
         my @envs = split(';', $vals{OnlyShowIn});
         if (@envs > 1) {
-            tag 'desktop-entry-limited-to-environments', $file;
+            $self->tag('desktop-entry-limited-to-environments', $file);
         }
     }
 
@@ -701,7 +737,7 @@ sub verify_desktop_file {
     if ($file =~ m,^usr/share/applications/, and defined $vals{'MimeType'}) {
         unless(defined $vals{'Exec'}
             and $vals{'Exec'} =~ m,(?:^|[^%])%[fFuU],){
-            tag 'desktop-mime-but-no-exec-code', $file;
+            $self->tag('desktop-mime-but-no-exec-code', $file);
         }
     }
 
@@ -714,7 +750,11 @@ sub verify_desktop_file {
 # false otherwise, and whose second member is the command (minus any leading
 # su-to-root wrapper).  Shared between the desktop and menu code.
 sub verify_cmd {
-    my ($file, $line, $exec, $pkg, $info) = @_;
+    my ($self, $file, $line, $exec) = @_;
+
+    my $pkg = $self->processable->name;
+    my $processable = $self->processable;
+
     my $location = ($line ? "$file:$line" : $file);
 
     # This routine handles su wrappers.  The option parsing here is ugly and
@@ -724,7 +764,7 @@ sub verify_cmd {
     my @com = split(' ', $exec);
     my $cmd;
     if ($com[0] and $com[0] eq '/usr/sbin/su-to-root') {
-        tag 'su-to-root-with-usr-sbin', $location;
+        $self->tag('su-to-root-with-usr-sbin', $location);
     }
     if (    $com[0]
         and $com[0] =~ m,^(?:/usr/s?bin/)?(su-to-root|gksu|kdesu|sux)$,) {
@@ -755,9 +795,9 @@ sub verify_cmd {
                 undef $wrapper;
             }
         }
-        tag 'su-wrapper-without--c', "$location $wrapper" unless $cmd;
+        $self->tag('su-wrapper-without--c', "$location $wrapper") unless $cmd;
         if ($wrapper && $wrapper !~ /su-to-root/ && $wrapper ne $pkg) {
-            tag 'su-wrapper-not-su-to-root', "$location $wrapper";
+            $self->tag('su-wrapper-not-su-to-root', "$location $wrapper");
         }
     } else {
         $cmd = $com[0];
@@ -768,10 +808,10 @@ sub verify_cmd {
     }
     my $okay = $cmd
       && ( $cmd =~ /^[\'\"]/
-        || $info->index($cmd_file)
+        || $processable->installed->lookup($cmd_file)
         || $cmd =~ m,^(/bin/)?sh,
         || $cmd =~ m,^(/usr/bin/)?sensible-(pager|editor|browser),
-        || any { $info->index($_ . $cmd) } @path);
+        || any { $processable->installed->lookup($_ . $cmd) } @path);
     return ($okay, $cmd_file);
 }
 
