@@ -2,7 +2,8 @@
 
 # somewhat of a misnomer -- it doesn't only check menus
 
-# Copyright (C) 1998 Christian Schwarz
+# Copyright © 1998 Christian Schwarz
+# Copyright © 2018 Chris Lamb <lamby@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,10 +28,11 @@ use warnings;
 use utf8;
 use autodie;
 
+use Path::Tiny;
+
 use Lintian::Data;
 use Lintian::Spelling
   qw(check_spelling check_spelling_picky $known_shells_regex);
-use Lintian::Util qw(file_is_encoded_in_non_utf8 strip);
 
 use Moo;
 use namespace::clean;
@@ -44,16 +46,16 @@ our %known_doc_base_formats = map { $_ => 1 }
 # Known fields for doc-base files.  The value is 1 for required fields and 0
 # for optional fields.
 our %KNOWN_DOCBASE_MAIN_FIELDS = (
-    'document' => 1,
-    'title'    => 1,
-    'section'  => 1,
-    'abstract' => 0,
-    'author'   => 0
+    'Document' => 1,
+    'Title'    => 1,
+    'Section'  => 1,
+    'Abstract' => 0,
+    'Author'   => 0
 );
 our %KNOWN_DOCBASE_FORMAT_FIELDS = (
-    'format'  => 1,
-    'files'   => 1,
-    'index'   => 0
+    'Format'  => 1,
+    'Files'   => 1,
+    'Index'   => 0
 );
 
 our $SECTIONS = Lintian::Data->new('doc-base/sections');
@@ -69,12 +71,12 @@ sub spelling_tag_emitter {
     };
 }
 
-sub files {
+sub visit_installed_files {
     my ($self, $file) = @_;
 
     if ($file->is_file) { # file checks
          # menu file?
-        if ($file =~ m,^usr/(lib|share)/menu/\S,o) { # correct permissions?
+        if ($file =~ m,^usr/(lib|share)/menu/\S,) { # correct permissions?
             if ($file->operm & 0111) {
                 $self->tag('executable-menu-file',
                     sprintf('%s %04o', $file, $file->operm));
@@ -83,19 +85,19 @@ sub files {
             return
               if $file =~ m,^usr/(?:lib|share)/menu/README$,;
 
-            if ($file =~ m,^usr/lib/,o) {
+            if ($file =~ m,^usr/lib/,) {
                 $self->tag('menu-file-in-usr-lib', $file);
             }
 
             $self->_set_menu_file($file->name);
 
-            if (    $file =~ m,usr/(?:lib|share)/menu/menu$,o
-                and $self->package ne 'menu') {
+            if (    $file =~ m,usr/(?:lib|share)/menu/menu$,
+                and $self->processable->name ne 'menu') {
                 $self->tag('bad-menu-file-name', $file);
             }
         }
         #menu-methods file?
-        elsif ($file =~ m,^etc/menu-methods/\S,o) {
+        elsif ($file =~ m,^etc/menu-methods/\S,) {
             #TODO: we should test if the menu-methods file
             # is made executable in the postinst as recommended by
             # the menu manual
@@ -107,27 +109,27 @@ sub files {
                 open(my $fd, '<', $file->unpacked_path);
                 while (<$fd>) {
                     chomp;
-                    if (m,^!include menu.h,o) {
+                    if (m,^!include menu.h,) {
                         $menumethod_includes_menu_h = 1;
                         last;
                     }
                 }
                 close($fd);
             }
-            $self->tag('menu-method-should-include-menu-h', $file)
+            $self->tag('menu-method-lacks-include', $file)
               unless $menumethod_includes_menu_h
-              or $self->package eq 'menu';
+              or $self->processable->name eq 'menu';
         }
         # package doc dir?
         elsif (
             $file =~ m{ \A usr/share/doc/(?:[^/]+/)?
                                  (.+\.(?:html|pdf))(?:\.gz)?
-                          \Z}xsmo
+                          \Z}xsm
         ) {
             my $name = $1;
-            unless ($name =~ m/^changelog\.html$/o
-                or $name =~ m/^README[.-]/o
-                or $name =~ m|examples|o) {
+            unless ($name =~ m/^changelog\.html$/
+                or $name =~ m/^README[.-]/
+                or $name =~ m/examples/) {
                 $self->_set_documentation(1);
             }
         }
@@ -139,7 +141,7 @@ sub files {
 sub installable {
     my ($self) = @_;
 
-    my $pkg = $self->package;
+    my $pkg = $self->processable->name;
     my $processable = $self->processable;
     my $group = $self->group;
 
@@ -256,29 +258,30 @@ sub installable {
 sub check_doc_base_file {
     my ($self, $dbpath, $all_files, $all_links) = @_;
 
-    my $pkg = $self->package;
+    my $pkg = $self->processable->name;
     my $group = $self->group;
 
     my $dbfile = $dbpath->basename;
-    my $line = file_is_encoded_in_non_utf8($dbpath->unpacked_path);
-    if ($line) {
-        $self->tag('doc-base-file-uses-obsolete-national-encoding',
-            "$dbfile:$line");
-    }
+
+    # another check complains about invalid encoding
+    return
+      unless ($dbpath->is_valid_utf8);
+
+    my $contents = $dbpath->decoded_utf8;
+    my @lines = split(/\n/, $contents);
 
     my $knownfields = \%KNOWN_DOCBASE_MAIN_FIELDS;
     my ($field, @vals);
     my %sawfields;        # local for each section of control file
     my %sawformats;       # global for control file
-    $line           = 0;  # global
+    my $line           = 0;  # global
 
-    open(my $fd, '<', $dbpath->unpacked_path);
-
-    while (<$fd>) {
-        chomp;
+    my $position = 1;
+    while (defined(my $string = shift @lines)) {
+        chomp $string;
 
         # New field.  check previous field, if we have any.
-        if (/^(\S+)\s*:\s*(.*)$/) {
+        if ($string =~ /^(\S+)\s*:\s*(.*)$/) {
             my (@new) = ($1, $2);
             if ($field) {
                 $self->check_doc_base_field(
@@ -287,21 +290,24 @@ sub check_doc_base_file {
                     $knownfields,$all_files, $all_links
                 );
             }
-            $field = lc $new[0];
+
+            $field = $new[0];
+
             @vals  = ($new[1]);
-            $line  = $.;
+            $line  = $position;
 
             # Continuation of previously defined field.
-        } elsif ($field && /^\s+\S/) {
-            push(@vals, $_);
+        } elsif ($field && $string =~ /^\s+\S/) {
+            push(@vals, $string);
 
             # All tags will be reported on the last continuation line of the
             # doc-base field.
-            $line  = $.;
+            $line  = $position;
 
             # Sections' separator.
-        } elsif (/^(\s*)$/) {
-            $self->tag('doc-base-file-separator-extra-whitespace',"$dbfile:$.")
+        } elsif ($string =~ /^(\s*)$/) {
+            $self->tag('doc-base-file-separator-extra-whitespace',
+                "$dbfile:$position")
               if $1;
             next unless $field; # skip successive empty lines
 
@@ -325,8 +331,11 @@ sub check_doc_base_file {
 
             # Everything else is a syntax error.
         } else {
-            $self->tag('doc-base-file-syntax-error', "$dbfile:$.");
+            $self->tag('doc-base-file-syntax-error', "$dbfile:$position");
         }
+
+    } continue {
+        ++$position;
     }
 
     # Check the last field/section of the control file.
@@ -341,11 +350,9 @@ sub check_doc_base_file {
     }
 
     # Make sure we saw at least one format.
-    $self->tag('doc-base-file-no-format-section', "$dbfile:$.")
+    $self->tag('doc-base-file-no-format-section', $dbfile)
       unless %sawformats;
 
-    close($fd);
-
     return;
 }
 
@@ -357,12 +364,12 @@ sub check_doc_base_field {
         $sawfields, $sawformats,$knownfields,$all_files, $all_links
     ) = @_;
 
-    my $pkg = $self->package;
+    my $pkg = $self->processable->name;
     my $group = $self->group;
 
     $self->tag('doc-base-file-unknown-field', "$dbfile:$line", $field)
       unless defined $knownfields->{$field};
-    $self->tag('doc-base-file-duplicated-field', "$dbfile:$line", $field)
+    $self->tag('duplicate-field-in-doc-base', "$dbfile:$line", $field)
       if $sawfields->{$field};
     $sawfields->{$field} = 1;
 
@@ -374,10 +381,10 @@ sub check_doc_base_field {
     # We skip without validating wildcard patterns containing character
     # classes since otherwise we'd need to deal with wildcards inside
     # character classes and aren't there yet.
-    if ($field eq 'index' or $field eq 'files') {
+    if ($field eq 'Index' or $field eq 'Files') {
         my @files = map { split(' ', $_) } @$vals;
 
-        if ($field eq 'index' && @files > 1) {
+        if ($field eq 'Index' && @files > 1) {
             $self->tag('doc-base-index-references-multiple-files',
                 "$dbfile:$line");
         }
@@ -394,7 +401,7 @@ sub check_doc_base_field {
             my $found;
             if ($realfile =~ /[*?]/) {
                 my $regex = quotemeta($realfile);
-                unless ($field eq 'index') {
+                unless ($field eq 'Index') {
                     next if $regex =~ /\[/;
                     $regex =~ s%\\\*%[^/]*%g;
                     $regex =~ s%\\\?%[^/]%g;
@@ -412,13 +419,16 @@ sub check_doc_base_field {
         undef @files;
 
         # Format field.
-    } elsif ($field eq 'format') {
+    } elsif ($field eq 'Format') {
         my $format = join(' ', @$vals);
-        strip($format);
+
+        # trim both ends
+        $format =~ s/^\s+|\s+$//g;
+
         $format = lc $format;
         $self->tag('doc-base-file-unknown-format', "$dbfile:$line", $format)
           unless $known_doc_base_formats{$format};
-        $self->tag('doc-base-file-duplicated-format', "$dbfile:$line", $format)
+        $self->tag('duplicate-format-in-doc-base', "$dbfile:$line", $format)
           if $sawformats->{$format};
         $sawformats->{$format} = 1;
 
@@ -426,7 +436,7 @@ sub check_doc_base_field {
         $sawformats->{' *current* '} = $format;
 
         # Document field.
-    } elsif ($field eq 'document') {
+    } elsif ($field eq 'Document') {
         $_ = join(' ', @$vals);
 
         $self->tag('doc-base-invalid-document-field', "$dbfile:$line", $_)
@@ -438,7 +448,7 @@ sub check_doc_base_field {
           unless $line == 1;
 
         # Title field.
-    } elsif ($field eq 'title') {
+    } elsif ($field eq 'Title') {
         if (@$vals) {
             my $stag_emitter
               = $self->spelling_tag_emitter(
@@ -453,7 +463,7 @@ sub check_doc_base_field {
         }
 
         # Section field.
-    } elsif ($field eq 'section') {
+    } elsif ($field eq 'Section') {
         $_ = join(' ', @$vals);
         unless ($SECTIONS->known($_)) {
             if (m,^App(?:lication)?s/(.+)$, and $SECTIONS->known($1)) {
@@ -467,7 +477,7 @@ sub check_doc_base_field {
         }
 
         # Abstract field.
-    } elsif ($field eq 'abstract') {
+    } elsif ($field eq 'Abstract') {
         # The three following variables are used for checking if the field is
         # correctly phrased.  We detect if each line (except for the first
         # line and lines containing single dot) of the field starts with the
@@ -490,19 +500,19 @@ sub check_doc_base_field {
         # Intentionally skipping the first line.
         for my $idx (1 .. $#{$vals}) {
             $_ = $vals->[$idx];
-            if (/manage\s+online\s+manuals\s.*Debian/o) {
+            if (/manage\s+online\s+manuals\s.*Debian/) {
                 $self->tag('doc-base-abstract-field-is-template',
                     "$dbfile:$line")
                   unless $pkg eq 'doc-base';
-            } elsif (/^(\s+)\.(\s*)$/o and ($1 ne ' ' or $2)) {
+            } elsif (/^(\s+)\.(\s*)$/ and ($1 ne ' ' or $2)) {
                 $self->tag(
                     'doc-base-abstract-field-separator-extra-whitespace',
                     "$dbfile:" . ($line - $#{$vals} + $idx));
-            } elsif (!$leadsp && /^(\s+)(\S)/o) {
+            } elsif (!$leadsp && /^(\s+)(\S)/) {
                 # The regexp should always match.
                 ($leadsp, $charafter) = ($1, $2);
                 $leadsp_ok = $leadsp eq ' ';
-            } elsif (!$leadsp_ok && /^(\s+)(\S)/o) {
+            } elsif (!$leadsp_ok && /^(\s+)(\S)/) {
                 # The regexp should always match.
                 undef $charafter if $charafter && $charafter ne $2;
                 $leadsp_ok = 1
@@ -539,16 +549,16 @@ sub check_doc_base_file_section {
     my ($self, $dbfile, $line, $sawfields, $sawformats, $knownfields) = @_;
 
     $self->tag('doc-base-file-no-format', "$dbfile:$line")
-      if ((defined $sawfields->{'files'} || defined $sawfields->{'index'})
-        && !(defined $sawfields->{'format'}));
+      if ((defined $sawfields->{'Files'} || defined $sawfields->{'Index'})
+        && !(defined $sawfields->{'Format'}));
 
     # The current format is set by check_doc_base_field.
-    if ($sawfields->{'format'}) {
+    if ($sawfields->{'Format'}) {
         my $format =  $sawformats->{' *current* '};
         $self->tag('doc-base-file-no-index', "$dbfile:$line")
           if ( $format
             && ($format eq 'html' || $format eq 'info')
-            && !$sawfields->{'index'});
+            && !$sawfields->{'Index'});
     }
     for my $field (sort keys %$knownfields) {
         $self->tag('doc-base-file-lacks-required-field',
@@ -643,7 +653,7 @@ sub delink {
 sub check_script {
     my ($self, $spath, $pres) = @_;
 
-    my $pkg = $self->package;
+    my $pkg = $self->processable->name;
 
     my ($no_check_menu, $no_check_installdocs, $interp);
 
@@ -671,7 +681,7 @@ sub check_script {
 
     while (<$fd>) {
         # skip comments
-        s/\#.*$//o;
+        s/\#.*$//;
 
         ##
         # update-menus will satisfy the checks that the menu file
@@ -679,9 +689,9 @@ sub check_script {
         ##
 
         # does the script check whether update-menus exists?
-        if (   /-x\s+\S*update-menus/o
-            or /(?:which|type)\s+update-menus/o
-            or /command\s+.*?update-menus/o) {
+        if (   /-x\s+\S*update-menus/
+            or /(?:which|type)\s+update-menus/
+            or /command\s+.*?update-menus/) {
             # yes, it does.
             $pres->{'checks-for-updatemenus'} = 1;
         }
@@ -707,9 +717,9 @@ sub check_script {
         }
 
         # does the script check whether install-docs exists?
-        if (   s/-x\s+\S*install-docs//o
-            or /(?:which|type)\s+install-docs/o
-            or s/command\s+.*?install-docs//o) {
+        if (   s/-x\s+\S*install-docs//
+            or /(?:which|type)\s+install-docs/
+            or s/command\s+.*?install-docs//) {
             # yes, it does.
             $pres->{'checks-for-installdocs'} = 1;
         }
