@@ -1,7 +1,7 @@
 # Hey emacs! This is a -*- Perl -*- script!
 # Lintian::Tag::TextUtil -- Perl utility functions for lintian
 
-# Copyright (C) 1998 Christian Schwarz and Richard Braakman
+# Copyright © 1998 Christian Schwarz and Richard Braakman
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,8 +31,6 @@ our @EXPORT_OK= qw(split_paragraphs wrap_paragraphs dtml_to_html dtml_to_text);
 # requires wrap() function
 use Text::Wrap;
 
-use Lintian::Util qw(strip);
-
 =head1 NAME
 
 Lintian::Tag::TextUtil -- text utility functions related to tags
@@ -105,13 +103,16 @@ sub split_paragraphs {
 
     my ($l,@o);
     while ($t) {
-        $t =~ s/^\.\n/\n/o;
+        $t =~ s/^\.\n/\n/;
         # starts with space or empty line?
-        if (($t =~ s/^([ \t][^\n]*)\n?//o) or ($t =~ s/^()\n//o)) {
+        if (($t =~ s/^([ \t][^\n]*)\n?//) or ($t =~ s/^()\n//)) {
             #FLUSH;
             if ($l) {
-                strip($l);
-                $l =~ s/\s++/ /go;
+
+                # trim both ends
+                $l =~ s/^\s+|\s+$//g;
+
+                $l =~ s/\s++/ /g;
                 push(@o,$l);
                 undef $l;
             }
@@ -119,7 +120,7 @@ sub split_paragraphs {
             push(@o,$1);
         }
         # normal line?
-        elsif ($t =~ s/^([^\n]*)\n?//o) {
+        elsif ($t =~ s/^([^\n]*)\n?//) {
             $l .= "$1 ";
         }
         # what else can happen?
@@ -129,8 +130,11 @@ sub split_paragraphs {
     }
     #FLUSH;
     if ($l) {
-        strip($l);
-        $l =~ s/\s++/ /go;
+
+        # trim both ends
+        $l =~ s/^\s+|\s+$//g;
+
+        $l =~ s/\s++/ /g;
         push(@o,$l);
         undef $l;
     }
@@ -149,18 +153,18 @@ sub dtml_to_html {
     my $pre=0;
     for $_ (@_) {
         s{\&maint\;}
-          {<a href=\"mailto:lintian-maint\@debian.org\">Lintian maintainer</a>}xsmo;
+          {<a href=\"mailto:lintian-maint\@debian.org\">Lintian maintainer</a>}xsm;
         s{\&debdev\;}
-          {<a href=\"mailto:debian-devel\@lists.debian.org\">debian-devel</a>}xsmo;
+          {<a href=\"mailto:debian-devel\@lists.debian.org\">debian-devel</a>}xsm;
 
         # empty line?
-        if (/^\s*$/o) {
+        if (/^\s*$/) {
             if ($pre) {
                 push(@o,"\n");
             }
         }
         # preformatted line?
-        elsif (/^\s/o) {
+        elsif (/^\s/) {
             if (not $pre) {
                 push(@o,'<pre>');
                 $pre=1;
@@ -171,7 +175,7 @@ sub dtml_to_html {
         else {
             if ($pre) {
                 my $last = pop @o;
-                $last =~ s,\n?$,</pre>\n,o;
+                $last =~ s,\n?$,</pre>\n,;
                 push @o, $last;
                 $pre=0;
             }
@@ -180,7 +184,7 @@ sub dtml_to_html {
     }
     if ($pre) {
         my $last = pop @o;
-        $last =~ s,\n?$,</pre>\n,o;
+        $last =~ s,\n?$,</pre>\n,;
         push @o, $last;
         $pre=0;
     }
@@ -195,26 +199,26 @@ sub dtml_to_html {
 sub dtml_to_text {
     for $_ (@_) {
         # substitute Lintian &tags;
-        s,&maint;,lintian-maint\@debian.org,go;
-        s,&debdev;,debian-devel\@lists.debian.org,go;
+        s,&maint;,lintian-maint\@debian.org,g;
+        s,&debdev;,debian-devel\@lists.debian.org,g;
 
         # substitute HTML <tags>
-        s,<i>,&lt;,go;
-        s,</i>,&gt;,go;
-        s,<[^>]+>,,go;
+        s,<i>,&lt;,g;
+        s,</i>,&gt;,g;
+        s,<[^>]+>,,g;
 
         # substitute HTML &tags;
-        s,&lt;,<,go;
-        s,&gt;,>,go;
-        s,&amp;,\&,go;
+        s,&lt;,<,g;
+        s,&gt;,>,g;
+        s,&amp;,\&,g;
 
         # preformatted?
-        if (not /^\s/o) {
+        if (not /^\s/) {
             # no.
 
-            s,\s\s+, ,go;
-            s,^ ,,o;
-            s, $,,o;
+            s,\s\s+, ,g;
+            s,^ ,,;
+            s, $,,;
         }
     }
 
