@@ -2,6 +2,7 @@
 # Lintian::Util -- Perl utility functions for lintian
 
 # Copyright © 1998 Christian Schwarz
+# Copyright © 2018-2019 Chris Lamb <lamby@debian.org>
 # Copyright © 2020 Felix Lechner
 #
 # This program is free software; you can redistribute it and/or modify
@@ -27,16 +28,7 @@ use warnings;
 use utf8;
 use autodie;
 
-use Carp qw(croak);
-use Cwd qw(abs_path);
-use Errno qw(ENOENT);
 use Exporter qw(import);
-use IO::Async::Loop;
-use IO::Async::Process;
-use Path::Tiny;
-use POSIX qw(sigprocmask SIG_BLOCK SIG_UNBLOCK SIG_SETMASK);
-
-use Lintian::Deb822Parser qw(read_dpkg_control parse_dpkg_control);
 
 # Force export as soon as possible, since some of the modules we load also
 # depend on us and the sequencing can cause things not to be exported
@@ -52,59 +44,54 @@ BEGIN {
     }
 
     @EXPORT_OK = (qw(
-          get_deb_info
-          get_dsc_info
           get_file_checksum
           get_file_digest
-          file_is_encoded_in_non_utf8
-          is_string_utf8_encoded
           do_fork
           run_cmd
-          safe_qx
-          strip
-          lstrip
-          rstrip
           copy_dir
           human_bytes
-          gunzip_file
           open_gz
-          gzip
           perm2oct
-          check_path
-          clean_env
+          locate_executable
           normalize_pkg_path
+          normalize_link_target
           is_ancestor_of
           locate_helper_tool
           drain_pipe
           drop_relative_prefix
           read_md5sums
+          version_from_changelog
           $PKGNAME_REGEX
           $PKGREPACK_REGEX
           $PKGVERSION_REGEX
     ));
 }
 
+use Carp qw(croak);
+use Cwd qw(abs_path);
 use Digest::MD5;
 use Digest::SHA;
 use Encode ();
+use Errno qw(ENOENT);
 use FileHandle;
+use List::MoreUtils qw(first_value);
+use Path::Tiny;
+use POSIX qw(sigprocmask SIG_BLOCK SIG_UNBLOCK SIG_SETMASK);
 use Scalar::Util qw(openhandle);
+use Unicode::UTF8 qw(valid_utf8);
 
+use Lintian::Deb822::File;
+use Lintian::Inspect::Changelog;
 use Lintian::Relation::Version qw(versions_equal versions_comparator);
 
 use constant EMPTY => q{};
 use constant SPACE => q{ };
-use constant COLON => q{:};
+use constant SLASH => q{/};
+use constant DOT => q{.};
+use constant DOUBLEDOT => q{..};
 use constant BACKSLASH => q{\\};
 use constant NEWLINE => qq{\n};
 
-# read up to 40kB at a time.  this happens to be 4096 "tar records"
-# (with a block-size of 512 and a block factor of 20, which appear to
-# be the defaults).  when we do full reads and writes of READ_SIZE (the
-# OS willing), the receiving end will never be with an incomplete
-# record.
-use constant READ_SIZE => 4096 * 20 * 512;
-
 # preload cache for common permission strings
 # call overhead o perm2oct was measurable on chromium-browser/32.0.1700.123-2
 # load time went from ~1.5s to ~0.1s; of 115363 paths, only 306 were uncached
@@ -122,12 +109,7 @@ Lintian::Util - Lintian utility functions
 
 =head1 SYNOPSIS
 
- use Lintian::Util qw(normalize_pkg_path);
-
- my $path = normalize_pkg_path('usr/bin/', '../lib/git-core/git-pull');
- if (defined $path) {
-    # ...
- }
+ use Lintian::Util;
 
 =head1 DESCRIPTION
 
@@ -147,7 +129,7 @@ is not anchored and does not enforce any "boundary" characters.
 
 =cut
 
-our $PKGNAME_REGEX = qr/[a-z0-9][-+\.a-z0-9]+/o;
+our $PKGNAME_REGEX = qr/[a-z0-9][-+\.a-z0-9]+/;
 
 =item $PKGREPACK_REGEX
 
@@ -157,7 +139,7 @@ be applied to the upstream portion (see #931846).
 
 =cut
 
-our $PKGREPACK_REGEX = qr/(dfsg|debian|ds|repack)/o;
+our $PKGREPACK_REGEX = qr/(dfsg|debian|ds|repack)/;
 
 =item $PKGVERSION_REGEX
 
@@ -171,7 +153,7 @@ our $PKGVERSION_REGEX = qr/
                  (?: \d+ : )?                # Optional epoch
                  [0-9][0-9A-Za-z.+:~]*       # Upstream version (with no hyphens)
                  (?: - [0-9A-Za-z.+:~]+ )*   # Optional debian revision (+ upstreams versions with hyphens)
-                          /xoa;
+                          /xa;
 
 =back
 
@@ -179,142 +161,6 @@ our $PKGVERSION_REGEX = qr/
 
 =over 4
 
-=item get_deb_info(DEBFILE)
-
-Extracts the control file from DEBFILE and returns it as a hashref.
-
-Basically, this is a fancy convenience for setting up an ar + tar pipe
-and passing said pipe to L</parse_dpkg_control(HANDLE[, FLAGS[, LINES]])>.
-
-DEBFILE must be an ar file containing a "control.tar.gz" member, which
-in turn should contain a "control" file.  If the "control" file is
-empty this will return an empty list.
-
-Note: the control file is only expected to have a single paragraph and
-thus only the first is returned (in the unlikely case that there are
-more than one).
-
-This function may fail with any of the messages that
-L</parse_dpkg_control> do.  It can also emit:
-
- "cannot fork to unpack %s: %s\n"
-
-=cut
-
-sub get_deb_info {
-    my ($path) = @_;
-
-    # dpkg-deb -f $file is very slow. Instead, we use ar and tar.
-
-    my $loop = IO::Async::Loop->new;
-
-    # get control tarball from deb
-    my $dpkgerror;
-    my $dpkgfuture = $loop->new_future;
-    my @dpkgcommand = ('dpkg-deb', '--ctrl-tarfile', $path);
-    my $dpkgprocess = IO::Async::Process->new(
-        command => [@dpkgcommand],
-        stdout => { via => 'pipe_read' },
-        stderr => { into => \$dpkgerror },
-        on_finish => sub {
-            my ($self, $exitcode) = @_;
-            my $status = ($exitcode >> 8);
-
-            if ($status) {
-                my $message= "Non-zero status $status from @dpkgcommand";
-                $message .= COLON . NEWLINE . $dpkgerror
-                  if length $dpkgerror;
-                $dpkgfuture->fail($message);
-                return;
-            }
-
-            $dpkgfuture->done("Done with @dpkgcommand");
-            return;
-        });
-
-    my $control;
-
-    # get the control file
-    my $tarerror;
-    my $tarfuture = $loop->new_future;
-    my @tarcommand = ('tar', '--wildcards', '-xO', '-f', '-', '*control');
-    my $tarprocess = IO::Async::Process->new(
-        command => [@tarcommand],
-        stdin => { via => 'pipe_write' },
-        stdout => { into => \$control },
-        stderr => { into => \$tarerror },
-        on_finish => sub {
-            my ($self, $exitcode) = @_;
-            my $status = ($exitcode >> 8);
-
-            if ($status) {
-                my $message = "Non-zero status $status from @tarcommand";
-                $message .= COLON . NEWLINE . $tarerror
-                  if length $tarerror;
-                $tarfuture->fail($message);
-                return;
-            }
-
-            $tarfuture->done("Done with @tarcommand");
-            return;
-        });
-
-    $tarprocess->stdin->configure(write_len => READ_SIZE);
-
-    $dpkgprocess->stdout->configure(
-        read_len => READ_SIZE,
-        on_read => sub {
-            my ($stream, $buffref, $eof) = @_;
-
-            if (length $$buffref) {
-                $tarprocess->stdin->write($$buffref);
-                $$buffref = EMPTY;
-            }
-
-            if ($eof) {
-                $tarprocess->stdin->close_when_empty;
-            }
-
-            return 0;
-        },
-    );
-
-    $loop->add($dpkgprocess);
-    $loop->add($tarprocess);
-
-    # awaits, and dies on failure with message from failed constituent
-    my $composite = Future->needs_all($dpkgfuture, $tarfuture);
-    $composite->get;
-
-    open(my $fh, '<', \$control);
-    my @data = parse_dpkg_control($fh);
-    close $fh;
-
-    return $data[0];
-}
-
-=item get_dsc_info (DSCFILE)
-
-Convenience function for reading dsc files.  It will read the DSCFILE
-using L</read_dpkg_control(FILE[, FLAGS[, LINES]])> and then return the
-first paragraph.  If the file has no paragraphs, C<undef> is returned
-instead.
-
-Note: the control file is only expected to have a single paragraph and
-thus only the first is returned (in the unlikely case that there are
-more than one).
-
-This function may fail with any of the messages that
-L</read_dpkg_control(FILE[, FLAGS[, LINES]])> do.
-
-=cut
-
-sub get_dsc_info {
-    my ($file) = @_;
-    my @data = read_dpkg_control($file);
-    return (defined($data[0])? $data[0] : undef);
-}
-
 =item drain_pipe(FD)
 
 Reads and discards any remaining contents from FD, which is assumed to
@@ -356,9 +202,9 @@ sub get_file_digest {
     my ($alg, $file) = @_;
     open(my $fd, '<', $file);
     my $digest;
-    if ($alg eq 'md5') {
+    if (lc($alg) eq 'md5') {
         $digest = Digest::MD5->new;
-    } elsif ($alg =~ /sha(\d+)/) {
+    } elsif (lc($alg) =~ /sha(\d+)/) {
         $digest = Digest::SHA->new($1);
     }
     $digest->addfile($fd);
@@ -383,54 +229,6 @@ sub get_file_checksum {
     return $digest->hexdigest;
 }
 
-=item is_string_utf8_encoded(STRING)
-
-Returns a truth value if STRING can be decoded as valid UTF-8.
-
-=cut
-
-{
-    my $decoder = Encode::find_encoding('UTF-8');
-    die('No UTF-8 decoder !?') unless ref($decoder);
-
-    sub is_string_utf8_encoded {
-        my ($str) = @_;
-        if ($str =~ m,\e[-!"\$%()*+./],) {
-            # ISO-2022
-            return 0;
-        }
-        eval {$decoder->decode($str, Encode::FB_CROAK);};
-        if ($@) {
-            # fail
-            return 0;
-        }
-        # pass
-        return 1;
-    }
-}
-
-=item file_is_encoded_in_non_utf8 (...)
-
-Undocumented
-
-=cut
-
-sub file_is_encoded_in_non_utf8 {
-    my ($file) = @_;
-
-    open(my $fd, '<', $file);
-    my $line = 0;
-    while (<$fd>) {
-        if (!is_string_utf8_encoded($_)) {
-            $line = $.;
-            last;
-        }
-    }
-    close($fd);
-
-    return $line;
-}
-
 =item do_fork()
 
 Overrides fork to reset signal handlers etc. in the child.
@@ -470,36 +268,6 @@ sub do_fork() {
     return $pid;
 }
 
-=item clean_env ([CLOC])
-
-Destructively cleans %ENV - removes all variables %ENV except a
-selected few whitelisted variables.
-
-The list of whitelisted %ENV variables are:
-
- PATH
- LC_ALL (*)
- TMPDIR
-
-(*) LC_ALL is a special case as clean_env will change its value to
-either "C.UTF-8" or "C" (if CLOC is given and a truth value).
-
-=cut
-
-sub clean_env {
-    my ($cloc) = @_;
-    my @whitelist = qw(PATH TMPDIR);
-    my %newenv
-      = map { exists $ENV{$_} ? ($_ => $ENV{$_}) : () } (@whitelist);
-    %ENV = %newenv;
-    $ENV{'LC_ALL'} = 'C.UTF-8';
-
-    if ($cloc) {
-        $ENV{LC_ALL} = 'C';
-    }
-    return;
-}
-
 =item perm2oct(PERM)
 
 Translates PERM to an octal permission.  PERM should be a string describing
@@ -653,54 +421,6 @@ sub run_cmd {
     return 1;
 }
 
-=item C<safe_qx(@cmd)>
-
-Emulates the C<qx()> operator by returning the captured output
-just like Capture::Tiny;
-
-Examples:
-
-  # Capture the output of a simple command
-  my $output = safe_qx('grep', 'some-pattern', 'path/to/file');
-
-=cut
-
-sub safe_qx {
-    my @command = @_;
-
-    my $loop = IO::Async::Loop->new;
-    my $future = $loop->new_future;
-    my $status;
-
-    $loop->run_child(
-        command => [@command],
-        on_finish => sub {
-            my ($pid, $exitcode, $stdout, $stderr) = @_;
-            $status = ($exitcode >> 8);
-
-            if ($status) {
-                my $message = "Command @command exited with status $status";
-                $message .= ": $stderr" if length $stderr;
-                $future->fail($message);
-                return;
-            }
-
-            $future->done($stdout);
-        });
-
-    $loop->await($future);
-
-    if ($future->is_failed) {
-        $? = $status;
-        return $future->failure;
-    }
-
-    $? = 0;
-
-    # will raise an exception in case of failure
-    return $future->get;
-}
-
 =item copy_dir (ARGS)
 
 Convenient way of calling I<cp -a ARGS>.
@@ -736,44 +456,6 @@ sub human_bytes {
     return $human;
 }
 
-=item gunzip_file (IN, OUT)
-
-Decompresses contents of the file IN and stores the contents in the
-file OUT.  IN is I<not> removed by this call.  On error, this function
-will cause a trappable error.
-
-=cut
-
-sub gunzip_file {
-    my ($inpath, $outpath) = @_;
-
-    my $loop = IO::Async::Loop->new;
-    my $future = $loop->new_future;
-
-    my @command = ('gzip', '--decompress', '--stdout', $inpath);
-    $loop->run_child(
-        command => [@command],
-        on_finish => sub {
-            my ($pid, $exitcode, $stdout, $stderr) = @_;
-            my $status = ($exitcode >> 8);
-
-            if ($status) {
-                my $message = "Command @command exited with status $status";
-                $message .= ": $stderr" if length $stderr;
-                $future->fail($message);
-                return;
-            }
-
-            path($outpath)->spew($stdout);
-            $future->done;
-        });
-
-    # will raise an exception in case of failure
-    $future->get;
-
-    return;
-}
-
 =item open_gz (FILE)
 
 Opens a handle that reads from the GZip compressed FILE.
@@ -792,6 +474,7 @@ sub __open_gz_pio {
     return $fd;
 }
 
+# Starting on 7/27/20, lintian depends on libperlio-gzip-perl
 # Fallback implementation of open_gz
 sub __open_gz_ext {
     my ($file) = @_;
@@ -799,55 +482,6 @@ sub __open_gz_ext {
     return $fd;
 }
 
-=item gzip (DATA, PATH)
-
-Compresses DATA using gzip and stores result in file located at PATH.
-
-=cut
-
-sub gzip {
-    my ($data, $path) = @_;
-
-    unlink($path)
-      if -e $path;
-
-    $data //= EMPTY;
-
-    my $loop = IO::Async::Loop->new;
-    my $future = $loop->new_future;
-    my $compressed;
-    my $stderr;
-
-    my @command = ('gzip', '--best', '--no-name', '--stdout');
-    my $process = IO::Async::Process->new(
-        command => [@command],
-        stdin => { from => $data },
-        stdout => { into => \$compressed },
-        stderr => { into => \$stderr },
-        on_finish => sub {
-            my ($self, $exitcode) = @_;
-            my $status = ($exitcode >> 8);
-
-            if ($status) {
-                my $message = "Command @command exited with status $status";
-                $message .= ": $stderr" if length $stderr;
-                $future->fail($message);
-                return;
-            }
-
-            $future->done('Done with command @command');
-            return;
-        });
-
-    $loop->add($process);
-
-    $future->get;
-
-    path($path)->spew($compressed // EMPTY);
-
-    return;
-}
-
 =item locate_helper_tool(TOOLNAME)
 
 Given the name of a helper tool, returns the path to it.  The tool
@@ -890,87 +524,20 @@ If the tool cannot be found, this sub will cause a trappable error.
     }
 }
 
-=item strip ([LINE])
-
-Strips whitespace from the beginning and the end of LINE and returns
-it.  If LINE is omitted, C<$_> will be used instead. Example
-
- @lines = map { strip } <$fd>;
-
-In void context, the input argument will be modified so it can be
-used as a replacement for chomp in some cases:
-
-  while ( my $line = <$fd> ) {
-    strip ($line);
-    # $line no longer has any leading or trailing whitespace
-  }
-
-Otherwise, a copy of the string is returned:
-
-  while ( my $orig = <$fd> ) {
-    my $stripped = strip ($orig);
-    if ($stripped ne $orig) {
-        # $orig had leading or/and trailing whitespace
-    }
-  }
-
-=item lstrip ([LINE])
-
-Like L<strip|/strip ([LINE])> but only strip leading whitespace.
-
-=item rstrip ([LINE])
-
-Like L<strip|/strip ([LINE])> but only strip trailing whitespace.
+=item locate_executable (CMD)
 
 =cut
 
-# prototype for default to $_
-sub strip (_) { ## no critic (Subroutines::RequireFinalReturn)
-    if (defined wantarray) {
-        # perl 5.14 s///r would have been useful here.
-        my ($arg) = @_;
-        $arg =~ s/^\s++//;
-        # unpack 'A*' is faster than s/\s++$//
-        return unpack('A*', $arg);
-    }
-    $_[0] =~ s/^\s++//;
-    $_[0] = unpack('A*', $_[0]);
-    # void context, so no return needed here.
-}
+sub locate_executable {
+    my ($command) = @_;
 
-# prototype for default to $_
-sub lstrip (_) { ## no critic (Subroutines::RequireFinalReturn)
-    if (defined wantarray) {
-        # perl 5.14 s///r would have been useful here.
-        my ($arg) = @_;
-        $arg =~ s/^\s++//;
-        return $arg;
-    }
-    $_[0] =~ s/^\s++//;
-    # void context, so no return needed here.
-}
+    return EMPTY
+      unless exists $ENV{PATH};
 
-{
-    no warnings qw(once);
-    *rstrip = \&Lintian::Deb822Parser::rstrip;
-}
+    my @folders =  grep { length } split(/:/, $ENV{PATH});
+    my $path = first_value { -x "$_/$command" } @folders;
 
-=item check_path (CMD)
-
-Returns 1 if CMD can be found in PATH (i.e. $ENV{PATH}) and is
-executable.  Otherwise, the function return 0.
-
-=cut
-
-sub check_path {
-    my $command = shift;
-
-    return 0 unless exists $ENV{PATH};
-    for my $element (split ':', $ENV{PATH}) {
-        next unless length $element;
-        return 1 if -f "$element/$command" and -x _;
-    }
-    return 0;
+    return ($path // EMPTY);
 }
 
 =item drop_relative_prefix(STRING)
@@ -988,6 +555,30 @@ sub drop_relative_prefix {
     return $copy;
 }
 
+=item version_from_changelog
+
+=cut
+
+sub version_from_changelog {
+    my ($package_path) = @_;
+
+    my $changelog_path = "$package_path/debian/changelog";
+
+    return EMPTY
+      unless -f $changelog_path;
+
+    my $contents = path($changelog_path)->slurp_utf8;
+    my $changelog = Lintian::Inspect::Changelog->new;
+
+    $changelog->parse($contents);
+    my @entries = @{$changelog->entries};
+
+    return $entries[0]->{'Version'}
+      if @entries;
+
+    return EMPTY;
+}
+
 =item signal_number2name(NUM)
 
 Given a number, returns the name of the signal (without leading
@@ -1034,15 +625,7 @@ normalize_pkg_path will return C<q{}> (i.e. the empty string) if PATH
 is normalized to the root dir and C<undef> if the path cannot be
 normalized without escaping the package root.
 
-Examples:
-  normalize_pkg_path('usr/share/java/../../../usr/share/ant/file')
-    eq 'usr/share/ant/file'
-  normalize_pkg_path('usr/..') eq q{};
-
- The following will return C<undef>:
-  normalize_pkg_path('usr/bin/../../../../etc/passwd')
-
-=item normalize_pkg_path(CURDIR, LINK_TARGET)
+=item normalize_link_target(CURDIR, LINK_TARGET)
 
 Normalize the path obtained by following a link with LINK_TARGET as
 its target from CURDIR as the current directory.  CURDIR is assumed to
@@ -1059,64 +642,52 @@ L<is_ancestor_of|Lintian::Util/is_ancestor_of(PARENTDIR, PATH)> for
 that.  If you must use this function, remember to check that the
 target is not a symlink (or if it is, that it can be resolved safely).
 
-Examples:
-
-  normalize_pkg_path('usr/share/java', '../ant/file') eq 'usr/share/ant/file'
-  normalize_pkg_path('usr/share/java', '../../../usr/share/ant/file')
-  normalize_pkg_path('usr/share/java', '/usr/share/ant/file')
-    eq 'usr/share/ant/file'
-  normalize_pkg_path('/usr/share/java', '/') eq q{};
-  normalize_pkg_path('/', 'usr/..') eq q{};
-
- The following will return C<undef>:
-  normalize_pkg_path('usr/bin', '../../../../etc/passwd')
-  normalize_pkg_path('usr/bin', '/../etc/passwd')
-
 =cut
 
+sub normalize_link_target {
+    my ($path, $target) = @_;
+
+    if (substr($target, 0, 1) eq SLASH) {
+        # Link is absolute
+        $path = $target;
+    } else {
+        # link is relative
+        $path = "$path/$target";
+    }
+
+    return normalize_pkg_path($path);
+}
+
 sub normalize_pkg_path {
-    my ($path, $dest) = @_;
-    my (@normalised, @queue);
+    my ($path) = @_;
 
-    if (@_ == 2) {
-        # We are doing CURDIR + LINK_TARGET
-        if (substr($dest, 0, 1) eq '/') {
-            # Link is absolute
-            # short circuit $dest eq '/' case.
-            return q{} if $dest eq '/';
-            $path = $dest;
-        } else {
-            # link is relative
-            $path = join('/', $path, $dest);
-        }
-    }
+    return EMPTY
+      if $path eq SLASH;
 
-    $path =~ s,//++,/,go;
-    $path =~ s,/$,,o;
-    $path =~ s,^/,,o;
+    my @dirty = split(m{/}, $path);
+    my @clean = grep { length } @dirty;
 
-    # Add all segments to the queue
-    @queue = split(m,/,o, $path);
+    my @final;
+    for my $component (@clean) {
 
-    # Loop through @queue and modify @normalised so that in the end of
-    # the loop, @normalised will contain the path that.
-    #
-    # Note that @normalised will be empty if we end in the root
-    # (e.g. '/' + 'usr' + '..' -> '/'), this is fine.
-    while (defined(my $target = shift(@queue))) {
-        if ($target eq '..') {
+        if ($component eq DOT) {
+            # do nothing
+
+        } elsif ($component eq DOUBLEDOT) {
             # are we out of bounds?
-            return unless @normalised;
-            # usr/share/java + '..' -> usr/share
-            pop(@normalised);
+            my $discard = pop @final;
+            return
+              unless defined $discard;
+
         } else {
-            # usr/share + java -> usr/share/java
-            # but usr/share + "." -> usr/share
-            push(@normalised, $target) if $target ne '.';
+            push(@final, $component);
         }
     }
-    return q{} unless @normalised;
-    return join('/', @normalised);
+
+    # empty if we end in the root
+    my $normalized = join(SLASH, @final);
+
+    return $normalized;
 }
 
 =item is_ancestor_of(PARENTDIR, PATH)
@@ -1198,28 +769,35 @@ sub unescape_md5sum_filename {
 sub read_md5sums {
     my ($text) = @_;
 
-    # make a local copy
-    my $buffer = $text;
-
     my %checksums;
     my @errors;
 
-  TRY_AGAIN:
+    my @lines = split(/\n/, $text);
 
     # start with checksum; processing style inspired by IO::Async::Stream
-    while ($buffer =~ s/^((?:\\)?\S{32}) [ *]//) {
+    while (defined(my $line = shift @lines)) {
+
+        next
+          unless length $line;
+
+        # make sure there are two spaces in between
+        $line =~ /^((?:\\)?\S{32})  (.*)$/;
 
         my $checksum = $1;
+        my $string = $2;
+
+        unless (length $checksum && length $string) {
+
+            push(@errors, "Odd text: $line");
+            next;
+        }
+
         my $problematic = 0;
 
         # leading slash in checksum indicates an escaped name
         $problematic = 1
           if $checksum =~ s{^\\}{};
 
-        # read up until the next newline
-        $buffer =~ s/^([^\n]+)\n//;
-        my $string = $1;
-
         my $path = unescape_md5sum_filename($string, $problematic);
 
         push(@errors, "Empty name for checksum $checksum")
@@ -1228,17 +806,6 @@ sub read_md5sums {
         $checksums{$path} = $checksum;
     }
 
-    if ($buffer =~ s/([^\n]*)(?:\n|\z)//) {
-        my $unreadable = $1;
-
-        # this skips empty lines
-        push(@errors, "Odd text: $unreadable")
-          if length $unreadable;
-
-        goto TRY_AGAIN
-          if length $buffer;
-    }
-
     return (\%checksums, \@errors);
 }
 
