@@ -1,5 +1,6 @@
-# Copyright (C) 1998 Richard Braakman
-# Copyright (C) 2012 Niels Thykier
+# Copyright © 1998 Richard Braakman
+# Copyright © 2012 Niels Thykier
+# Copyright © 2018 Chris Lamb <lamby@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -73,13 +74,13 @@ use File::Basename qw(basename);
 use File::Find ();
 use List::MoreUtils qw{any};
 use Path::Tiny;
+use Unicode::UTF8 qw(valid_utf8 decode_utf8);
 
 use Lintian::Spelling qw(check_spelling);
 use Lintian::Data;
-use Lintian::Deb822Parser qw(read_dpkg_control);
+use Lintian::Deb822::Parser qw(parse_dpkg_control_string);
 use Lintian::Profile;
 use Lintian::Tag::Info;
-use Lintian::Util qw(is_string_utf8_encoded);
 
 # We want data files loaded early to avoid problems with missing data
 # files ending up in releases (like in 2.5.17 and 2.5.18).
@@ -161,7 +162,13 @@ sub test_check_desc {
     my @descs = map { _find_check($find_opt, $_) } @dirs;
     foreach my $desc_file (@descs) {
         my ($header, @tagpara);
-        eval {($header, @tagpara) = read_dpkg_control($desc_file);};
+        my $bytes = path($desc_file)->slurp;
+        $builder->ok(valid_utf8($bytes),
+            "File $desc_file does not use a national encoding.");
+        next
+          unless valid_utf8($bytes);
+        my $contents = decode_utf8($bytes);
+        eval {($header, @tagpara) = parse_dpkg_control_string($contents);};
         if (my $err = $@) {
             $err =~ s/ at .*? line \d+\s*\n//;
             $builder->ok(0, "Cannot parse $desc_file");
@@ -169,9 +176,8 @@ sub test_check_desc {
             next;
         }
         my $content_type = 'Check';
-        my $cname = $header->{'check-script'}//'';
-        my $ctype = $header->{'type'} // '';
-        my $needs = $header->{'needs-info'} // '';
+        my $cname = $header->{'Check-Script'}//'';
+        my $ctype = $header->{'Type'} // '';
         my $i = 1; # paragraph counter.
         $builder->ok(1, "Can parse check $desc_file");
 
@@ -190,8 +196,8 @@ sub test_check_desc {
         } elsif ($builder->isnt_eq($ctype, '', "$cname has a type")) {
             my @bad;
             # new lines are not allowed, map them to "\\n" for readability.
-            $ctype =~ s/\n/\\n/go;
-            foreach my $type (split m/\s*+,\s*+/o, $ctype) {
+            $ctype =~ s/\n/\\n/g;
+            foreach my $type (split /\s*+,\s*+/, $ctype) {
                 push @bad, $type unless exists $check_types{$type};
             }
             $builder->is_eq(join(', ', @bad),'',"The type of $cname is valid");
@@ -202,9 +208,9 @@ sub test_check_desc {
         }
 
         foreach my $tpara (@tagpara) {
-            my $tag = $tpara->{'tag'}//'';
-            my $severity = $tpara->{'severity'}//'';
-            my $info = $tpara->{'info'} // '';
+            my $tag = $tpara->{'Tag'}//'';
+            my $severity = $tpara->{'Severity'}//'';
+            my $info = $tpara->{'Info'} // '';
             my (@htmltags, %seen);
 
             $i++;
@@ -251,10 +257,8 @@ sub test_check_desc {
                 'Tag info uses only two spaces after a full stop')
               or $builder->diag("$content_type $cname: $tag ($1)\n");
 
-            $builder->ok(
-                is_string_utf8_encoded($info),
-                'Tag info must be written in UTF-8'
-            ) or $builder->diag("$content_type $cname: $tag\n");
+            $builder->ok(valid_utf8($info),'Tag info must be written in UTF-8')
+              or $builder->diag("$content_type $cname: $tag\n");
 
             # Check the tag info for unescaped <> or for unknown tags (which
             # probably indicate the same thing).
@@ -271,8 +275,8 @@ sub test_check_desc {
                 'Tag info has no stray angle brackets')
               or $builder->diag("$content_type $cname: $tag\n");
 
-            if ($tpara->{'ref'}) {
-                my @issues = _check_reference($tpara->{'ref'});
+            if ($tpara->{'Ref'}) {
+                my @issues = _check_reference($tpara->{'Ref'});
                 my $text = join("\n\t", @issues);
                 $builder->ok(!@issues, 'Proper references are used')
                   or $builder->diag("$content_type $cname: $tag\n\t$text");
@@ -316,7 +320,7 @@ sub test_load_profiles {
         my $profname = $File::Find::name;
 
         return
-          unless $profname =~ s/\.profile$//o;
+          unless $profname =~ s/\.profile$//;
         $profname =~ s,^$sre,,;
 
         my $profile = Lintian::Profile->new;
@@ -429,8 +433,8 @@ sub test_load_checks {
             next;
         }
 
-        $ppkg =~ s,[-.],_,go;
-        $ppkg =~ s,/,::,go;
+        $ppkg =~ s,[-.],_,g;
+        $ppkg =~ s,/,::,g;
         $ppkg = "Lintian::$ppkg";
 
         if ($ppkg->can('run') && !$ppkg->DOES('Lintian::Check')) {
@@ -595,7 +599,7 @@ profile has been loaded, 'debian/main' will be loaded.
 
 INC is a list of extra "include dirs" (or Lintian "roots") to be used
 for finding the profile.  If not specified, it defaults to
-I<$ENV{'LINTIAN_TEST_ROOT'}> and I</usr/share/lintian> (in order).
+I<$ENV{'LINTIAN_BASE'}> and I</usr/share/lintian> (in order).
 INC is ignored if a profile has already been loaded.
 
 CAVEAT: Only one profile can be loaded in a given test.  Once a
@@ -620,17 +624,12 @@ sub load_profile_for_test {
      # do just fine...
     $profname ||= 'debian/main';
 
-    my $location
-      = $ENV{'LINTIAN_ROOT'} ? $ENV{'LINTIAN_ROOT'} : '/usr/share/lintian';
-    push(@inc, $location)
-      unless @inc;
-
     $PROFILE = Lintian::Profile->new;
-    $PROFILE->load($profname, \@inc);
+    $PROFILE->load($profname, [@inc, $ENV{'LINTIAN_BASE'}]);
 
     Lintian::Data->set_vendor($PROFILE);
     $ENV{'LINTIAN_HELPER_DIRS'} = join(':', map { "$_/helpers" } @inc);
-    $ENV{'LINTIAN_INCLUDE_DIRS'} = join(':', @inc);
+    $ENV{'LINTIAN_CONFIG_DIRS'} = join(':', @inc);
     return;
 }
 
@@ -757,7 +756,7 @@ files/symlinks beneath it.
 
     my %SPECIAL_PATHS = (
         'docs-examples' => ['doc/examples/checks'],
-        'test-scripts' => [qw(t/scripts t/bin t/templates)],
+        'test-scripts' => [qw(t/scripts t/templates)],
     );
 
     sub program_name_to_perl_paths {
