@@ -1,4 +1,5 @@
 # Copyright © 2018 Felix Lechner
+# Copyright © 2019 Chris Lamb <lamby@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -36,10 +37,10 @@ Generic test runner for all Lintian test suites
 
 =cut
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
-use v5.10;
 
 use Exporter qw(import);
 
@@ -65,16 +66,17 @@ use Test::More;
 use Text::Diff;
 use Try::Tiny;
 
+use Lintian::Deb822::File;
 use Lintian::Profile;
 
 use Test::Lintian::ConfigFile qw(read_config);
 use Test::Lintian::Helper qw(rfc822date);
 use Test::Lintian::Hooks
-  qw(find_missing_prerequisites run_lintian sed_hook sort_lines calibrate);
-use Test::Lintian::Prepare qw(early_logpath);
+  qw(find_missing_prerequisites sed_hook sort_lines calibrate);
 use Test::Lintian::Output::Universal qw(get_tagnames order);
 
 use constant SPACE => q{ };
+use constant SLASH => q{/};
 use constant EMPTY => q{};
 use constant NEWLINE => qq{\n};
 use constant YES => q{yes};
@@ -106,12 +108,12 @@ sub logged_runner {
     my $files = read_config($runfiles);
 
     # set path to logfile
-    my $betterlogpath = "$runpath/$files->{log}";
+    my $logpath = $runpath . SLASH . $files->unfolded_value('Log');
 
     my $log = capture_merged {
         try {
             # call runner
-            runner($runpath, $betterlogpath)
+            runner($runpath, $logpath)
 
         }catch {
             # catch any error
@@ -119,18 +121,11 @@ sub logged_runner {
         };
     };
 
-    # delete old runner log
-    unlink $betterlogpath if -f $betterlogpath;
-
-    # move the early log for directory preparation to position of runner log
-    my $earlylogpath = early_logpath($runpath);
-    move($earlylogpath, $betterlogpath) if -f $earlylogpath;
-
     # append runner log to population log
-    path($betterlogpath)->append_utf8($log) if length $log;
+    path($logpath)->append_utf8($log) if length $log;
 
     # add error if there was one
-    path($betterlogpath)->append_utf8($error) if length $error;
+    path($logpath)->append_utf8($error) if length $error;
 
     # print log and die on error
     if ($error) {
@@ -154,9 +149,6 @@ sub runner {
     # set a predictable locale
     $ENV{'LC_ALL'} = 'C';
 
-    # many tests create files via debian/rules
-    umask(022);
-
     say EMPTY;
     say '------- Runner starts here -------';
 
@@ -174,7 +166,8 @@ sub runner {
     my $spec_epoch = stat($runfiles)->mtime;
 
     # read dynamic case data
-    my $rundescpath = "$runpath/$files->{test_specification}";
+    my $rundescpath
+      = $runpath . SLASH . $files->unfolded_value('Test-Specification');
     my $testcase = read_config($rundescpath);
 
     # get data age
@@ -201,11 +194,11 @@ sub runner {
     my $lintian_epoch = $ENV{'LINTIAN_EPOCH'}//time;
     say 'Lintian modified on  : '. rfc822date($lintian_epoch);
 
+    my $testname = $testcase->unfolded_value('Testname');
     # name of encapsulating directory should be that of test
     my $expected_name = path($runpath)->basename;
-    die
-"Test in $runpath is called $testcase->{testname} instead of $expected_name"
-      if ($testcase->{testname} ne $expected_name);
+    die"Test in $runpath is called $testname instead of $expected_name"
+      unless $testname eq $expected_name;
 
     # skip test if marked
     my $skipfile = "$runpath/skip";
@@ -227,7 +220,7 @@ sub runner {
         say 'DEB_HOST_ARCH is not set.';
         BAIL_OUT('DEB_HOST_ARCH is not set.');
     }
-    my $platforms = $testcase->{test_architectures};
+    my $platforms = $testcase->unfolded_value('Test-Architectures');
     if ($platforms ne 'any') {
         my @wildcards = split(SPACE, $platforms);
         my @matches= map {
@@ -239,26 +232,40 @@ sub runner {
         }
     }
 
+    plan skip_all => 'No package found'
+      unless -f "$runpath/subject";
+
     # set the testing plan
     plan tests => 1;
 
+    my $subject = path("$runpath/subject")->realpath;
+
     # get lintian subject
     die 'Could not get subject of Lintian examination.'
-      unless exists $testcase->{build_product};
-    my $subject = "$runpath/$testcase->{build_product}";
+      unless -f $subject;
 
     # run lintian
-    $ENV{'LINTIAN_COVERAGE'}.= ",-db,./cover_db-$testcase->{testname}"
+    $ENV{'LINTIAN_COVERAGE'}.= ",-db,./cover_db-$testname"
       if exists $ENV{'LINTIAN_COVERAGE'};
 
+    my $lintian_command_line
+      = $testcase->unfolded_value('Lintian-Command-Line');
     my $command
-      = "cd $runpath; $ENV{'LINTIAN_FRONTEND'} $testcase->{lintian_command_line} $testcase->{subject}";
+      = "cd $runpath; $ENV{'LINTIAN_UNDER_TEST'} $lintian_command_line $subject";
     say $command;
     my ($output, $status) = capture_merged { system($command); };
     $status = ($status >> 8) & 255;
 
-    die "$command exited with status $status" . NEWLINE . $output
-      unless $status == 0 || $status == 1;
+    say "$command exited with status $status.";
+    say $output if $status == 1;
+
+    my $expected_status = $testcase->unfolded_value('Exit-Status');
+
+    my @errors;
+    push(@errors,
+        "Exit code $status differs from expected value $expected_status.")
+      if $testcase->exists('Exit-Status')
+      && $status != $expected_status;
 
     # filter out some warnings if running under coverage
     my @lines = split(/\n/, $output);
@@ -276,20 +283,23 @@ sub runner {
     $output .= $_ . NEWLINE for @lines;
 
     die 'No match strategy defined'
-      unless length $testcase->{match_strategy};
+      unless $testcase->exists('Match-Strategy');
+
+    my $match_strategy = $testcase->unfolded_value('Match-Strategy');
+
+    if ($match_strategy eq 'literal') {
+        push(@errors, check_literal($testcase, $runpath, $output));
+
+    } elsif ($match_strategy eq 'tags') {
+        push(@errors, check_tags($testcase, $runpath, $output));
 
-    my @errors;
-    if ($testcase->{match_strategy} eq 'literal') {
-        @errors = check_literal($testcase, $runpath, $output);
-    } elsif ($testcase->{match_strategy} eq 'tags') {
-        @errors = check_tags($testcase, $runpath, $output);
     } else {
-        die "Unknown match strategy $testcase->{match_strategy}.";
+        die "Unknown match strategy $match_strategy.";
     }
 
     my $okay = !(scalar @errors);
 
-    if($testcase->{todo} eq 'yes') {
+    if($testcase->unfolded_value('Todo') eq 'yes') {
       TODO: {
             local $TODO = 'Test marked as TODO.';
             ok($okay, 'Lintian passes for test marked TODO.');
@@ -297,13 +307,17 @@ sub runner {
         return;
     }
 
-    ok($okay, "Lintian passes for $testcase->{testname}");
-
     diag $_ . NEWLINE for @errors;
 
+    ok($okay, "Lintian passes for $testname");
+
     return;
 }
 
+=item check_literal
+
+=cut
+
 sub check_literal {
     my ($testcase, $runpath, $output) = @_;
 
@@ -317,7 +331,7 @@ sub check_literal {
 
     # run a sed-script if it exists
     my $actual = "$runpath/literal.actual.parsed";
-    my $script = "$runpath/post_test";
+    my $script = "$runpath/post-test";
     if(-f $script) {
         sed_hook($script, $raw, $actual);
     } else {
@@ -328,6 +342,10 @@ sub check_literal {
     return check_result($testcase, $runpath, $expected, $actual);
 }
 
+=item check_tags
+
+=cut
+
 sub check_tags {
     my ($testcase, $runpath, $output) = @_;
 
@@ -341,7 +359,7 @@ sub check_tags {
 
     # run a sed-script if it exists
     my $actual = "$runpath/tags.actual.parsed";
-    my $sedscript = "$runpath/post_test";
+    my $sedscript = "$runpath/post-test";
     if(-f $sedscript) {
         sed_hook($sedscript, $raw, $actual);
     } else {
@@ -351,7 +369,7 @@ sub check_tags {
 
     # calibrate tags; may write to $actual
     my $calibrated = "$runpath/tags.specified.calibrated";
-    my $calscript = "$runpath/test_calibration";
+    my $calscript = "$runpath/test-calibration";
     if(-x $calscript) {
         calibrate($calscript, $actual, $expected, $calibrated);
     } else {
@@ -383,7 +401,9 @@ sub check_result {
     push(@actuallines, NEWLINE)
       unless @actuallines;
 
-    if ($testcase->{match_strategy} eq 'tags') {
+    my $match_strategy = $testcase->unfolded_value('Match-Strategy');
+
+    if ($match_strategy eq 'tags') {
         @expectedlines
           = reverse sort { order($a) cmp order($b) } @expectedlines;
         @actuallines
@@ -398,10 +418,10 @@ sub check_result {
 
     if(@difflines) {
 
-        if ($testcase->{match_strategy} eq 'literal') {
+        if ($match_strategy eq 'literal') {
             push(@errors, 'Literal output does not match');
 
-        } elsif ($testcase->{match_strategy} eq 'tags') {
+        } elsif ($match_strategy eq 'tags') {
 
             push(@errors, 'Tags do not match');
 
@@ -411,7 +431,7 @@ sub check_result {
             path("$runpath/tagdiff")->spew($tagdiff // EMPTY);
 
         } else {
-            die "Unknown match strategy $testcase->{match_strategy}.";
+            die "Unknown match strategy $match_strategy.";
         }
 
         push(@errors, EMPTY);
@@ -425,7 +445,7 @@ sub check_result {
 
     # stop if the test is not about tags
     return @errors
-      unless $testcase->{match_strategy} eq 'tags';
+      unless $match_strategy eq 'tags';
 
     # get expected tags
     my @expected = sort +get_tagnames($expectedpath);
@@ -435,14 +455,16 @@ sub check_result {
     # look out for tags being tested
     my @related;
 
-    if (length $testcase->{check} && $testcase->{check} ne 'all') {
+    if (   $testcase->exists('Check')
+        && $testcase->unfolded_value('Check') ne 'all') {
 
-        my $profile = Lintian::Profile->new(undef, [$ENV{LINTIAN_ROOT}]);
+        my $profile = Lintian::Profile->new;
+        $profile->load(undef, [$ENV{LINTIAN_BASE}]);
 
         # use tags related to checks declared
-        my @checks = split(SPACE, $testcase->{check});
+        my @checks = $testcase->trimmed_list('Check');
         foreach my $check (@checks) {
-            my $checkscript = $profile->get_script($check);
+            my $checkscript = $profile->get_checkinfo($check);
             die "Unknown Lintian check $check"
               unless defined $checkscript;
 
@@ -496,6 +518,14 @@ sub check_result {
 
 =back
 
+=head1 AUTHOR
+
+Originally written by Felix Lechner <felix.lechner@lease-up.com> for Lintian.
+
+=head1 SEE ALSO
+
+lintian(1)
+
 =cut
 
 1;
